/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2024, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
package net.sourceforge.plantuml.quantization;

import java.util.Set;

public interface ColorQuantizer {
	/**
	 * Quantize the given set of colors, returning a set no larger than
	 * {@code maxColors}.
	 *
	 * <p>
	 * The intent is to pick a set of colors which are representative of the
	 * original color set, but no specific guarantees are made.
	 *
	 * @param originalColors the colors in the original image
	 * @param maxColorCount  the maximum number of colors to allow
	 * @return a quantized collection of colors no larger than {@code maxColors}
	 */
	public Set<QColor> quantize(Multiset<QColor> originalColors, int maxColorCount);
}
