/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2024, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
package net.sourceforge.plantuml.quantization;

import java.util.Collection;
import java.util.Set;

/**
 * A collection which permits duplicates, and provides methods adding/removing
 * several counts of an element.
 *
 * @param <E> the element type
 */
public interface Multiset<E> extends Collection<E> {
	/**
	 * Add n counts of an element.
	 *
	 * @param element the element to add
	 * @param n       how many to add
	 */
	public void add(E element, int n);

	/**
	 * Remove up to n counts of an element.
	 *
	 * @param element the element the remove
	 * @param n       how many to remove
	 * @return the number of elements removed
	 */
	public int remove(Object element, int n);

	public int count(Object element);

	public Set<E> getDistinctElements();
}
